# -*- coding: utf-8 -*-

# Copyright (c) 2011-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

import binascii
import time
import logging
import os
import re
from urllib.parse import urlsplit
from socket import gethostbyname, gaierror
import importlib

import simplejson as json
from ipcalc import IP, Network
from Crypto.Cipher import AES

import zope.event.classhandler
from pyramid.config import Configurator
from pyramid.interfaces import IStaticURLInfo
from pyramid.httpexceptions import HTTPException
from pyramid.path import AssetResolver
import pyramid.security
from pyramid_mako import add_mako_renderer
from papyrus.renderers import GeoJSON
from sqlalchemy.orm import Session

import c2cwsgiutils
import c2cwsgiutils.db
import c2cwsgiutils.index
from c2cwsgiutils.health_check import HealthCheck

import c2cgeoportal_commons.models
from c2cgeoportal_geoportal.lib.xsd import XSD
from c2cgeoportal_geoportal.lib import dbreflection, caching, \
    C2CPregenerator, MultiDomainStaticURLInfo, checker, check_collector
import c2cgeoportal_geoportal.views


LOG = logging.getLogger(__name__)

# Header predicate to accept only JSON content
JSON_CONTENT_TYPE = "Content-Type:application/json"
GEOJSON_CONTENT_TYPE = r'Content-Type:application/geo\+json'


class DecimalJSON:
    def __call__(self, info):
        def _render(value, system):
            ret = json.dumps(value, use_decimal=True)
            request = system.get("request")
            if request is not None:
                request.response.content_type = "application/json"
            return ret
        return _render


class AssetRendererFactory:
    def __init__(self, info):
        del info  # unused
        self.resolver = AssetResolver('c2cgeoportal_geoportal')

    def __call__(self, value, system):
        asset = self.resolver.resolve(system['renderer_name'])
        return asset.stream().read()


INTERFACE_TYPE_NGEO = "ngeo"


def add_interface(
    config, interface_name="desktop", interface_type=INTERFACE_TYPE_NGEO, default=False, **kwargs
):  # pragma: no cover
    del interface_type  # unused
    route = "/" if default else "/{}".format(interface_name)
    add_interface_ngeo(
        config,
        route_name=interface_name,
        route=route,
        renderer="{}_geoportal:static-ngeo/build/{}.html".format(
            config.get_settings()['package'], interface_name
        ),
        **kwargs
    )


def add_interface_ngeo(config, route_name, route, renderer=None, permission=None):  # pragma: no cover
    # Cannot be at the header to do not load the model too early
    from c2cgeoportal_geoportal.views.entry import Entry

    config.add_route(route_name, route, request_method="GET")
    config.add_view(
        Entry,
        attr="get_ngeo_index_vars",
        route_name=route_name,
        renderer=renderer,
        permission=permission
    )
    # Permalink theme: recover the theme for generating custom viewer.js url
    config.add_route(
        "{}theme".format(route_name),
        "{}{}theme/{{themes}}".format(route, "" if route[-1] == "/" else "/"),
        request_method="GET",
    )
    config.add_view(
        Entry,
        attr="get_ngeo_index_vars",
        route_name="{}theme".format(route_name),
        renderer=renderer,
        permission=permission
    )


def add_admin_interface(config):
    if config.get_settings().get('enable_admin_interface', False):
        config.add_request_method(
            # pylint: disable=not-callable
            lambda request: c2cgeoportal_commons.models.DBSession(), 'dbsession', reify=True
        )
        config.add_view(c2cgeoportal_geoportal.views.add_ending_slash, route_name='admin_add_ending_slash')
        config.add_route('admin_add_ending_slash', '/admin', request_method='GET')
        config.include('c2cgeoportal_admin', route_prefix='/admin')


def locale_negotiator(request):
    lang = request.params.get("lang")
    if lang is None:
        lang = request.cookies.get('_LOCALE_')
    else:
        request.response.set_cookie('_LOCALE_', lang)
    if lang is None:
        # if best_match returns None then use the default_locale_name configuration variable
        return request.accept_language.best_match(
            request.registry.settings.get("available_locale_names"),
            default_match=request.registry.settings.get("default_locale_name"))
    return lang


def _match_url_start(reference, value):
    """
    Checks that the val URL starts like the ref URL.
    """
    reference_parts = reference.rstrip("/").split("/")
    value_parts = value[0:len(reference_parts)]
    return reference_parts == value_parts


def is_valid_referer(request, settings=None):
    if request.referer is not None:
        referer = urlsplit(request.referer)._replace(query="", fragment="").geturl().rstrip("/").split("/")
        if settings is None:
            settings = request.registry.settings
        list_ = settings.get("authorized_referers", [])
        return any(_match_url_start(e, referer) for e in list_)
    else:
        return True


def create_get_user_from_request(settings):
    def get_user_from_request(request, username=None):
        """ Return the User object for the request.

        Return ``None`` if:
        * user is anonymous
        * it does not exist in the database
        * the referer is invalid
        """
        from c2cgeoportal_commons.models import DBSession
        from c2cgeoportal_commons.models.static import User

        try:
            if "auth" in request.params:
                auth_enc = request.params.get("auth")

                if auth_enc is not None:
                    urllogin = request.registry.settings.get("urllogin", {})
                    aeskey = urllogin.get("aes_key")
                    if aeskey is None:  # pragma: nocover
                        raise Exception("urllogin is not configured")
                    now = int(time.time())
                    data = binascii.unhexlify(auth_enc.encode('ascii'))
                    nonce = data[0:16]
                    tag = data[16:32]
                    ciphertext = data[32:]
                    cipher = AES.new(aeskey.encode("ascii"), AES.MODE_EAX, nonce)
                    auth = json.loads(cipher.decrypt_and_verify(ciphertext, tag).decode("utf-8"))

                    if "t" in auth and "u" in auth and "p" in auth:
                        timestamp = int(auth["t"])
                        if now < timestamp and request.registry.validate_user(
                            request, auth["u"], auth["p"]
                        ):
                            headers = pyramid.security.remember(request, auth["u"])
                            request.response.headerlist.extend(headers)
        except Exception as e:
            LOG.error("URL login error: %s.", e, exc_info=True)

        if not hasattr(request, "is_valid_referer"):
            request.is_valid_referer = is_valid_referer(request, settings)
        if not request.is_valid_referer:
            LOG.debug("Invalid referer for %s: %s", request.path_qs, repr(request.referer))
            return None

        if not hasattr(request, "user_"):
            request.user_ = None
            if username is None:
                username = request.authenticated_userid
            if username is not None:
                # We know we will need the role object of the
                # user so we use joined loading
                request.user_ = DBSession.query(User) \
                    .filter_by(username=username) \
                    .first()

        return request.user_
    return get_user_from_request


def set_user_validator(config, user_validator):
    """ Call this function to register a user validator function.

    The validator function is passed three arguments: ``request``,
    ``username``, and ``password``. The function should return the
    user name if the credentials are valid, and ``None`` otherwise.

    The validator should not do the actual authentication operation
    by calling ``remember``, this is handled by the ``login`` view.
    """
    def register():
        config.registry.validate_user = user_validator
    config.action("user_validator", register)


def default_user_validator(request, username, password):
    """
    Validate the username/password. This is c2cgeoportal's
    default user validator.
    Return None if we are anonymous, the string to remember otherwise.
    """
    del request  # unused
    from c2cgeoportal_commons.models import DBSession
    from c2cgeoportal_commons.models.static import User
    user = DBSession.query(User).filter_by(username=username).first()
    if user is None:
        LOG.info('Unknow user "%s" tried to log in', username)
        return None
    if user.deactivated:
        LOG.info('Deactivated user "%s" tried to log in', username)
        return None
    if user.expired():
        LOG.info('Expired user %s tried to log in', username)
        return None
    if not user.validate_password(password):
        LOG.info('User "%s" tried to log in with bad credentials', username)
        return None
    return username


class OgcproxyRoutePredicate:
    """ Serve as a custom route predicate function for ogcproxy.
    We do not want the OGC proxy to be used to reach the app's
    mapserv script. We just return False if the url includes
    "mapserv". It is rather drastic, but works for us. """

    def __init__(self, val, config):
        del val  # unused
        del config  # unused
        self.private_networks = [
            Network("127.0.0.0/8"),
            Network("10.0.0.0/8"),
            Network("172.16.0.0/12"),
            Network("192.168.0.0/16"),
        ]

    def __call__(self, context, request):
        url = request.params.get("url")
        if url is None:
            return False

        parts = urlsplit(url)
        try:
            ip = IP(gethostbyname(parts.netloc))
        except gaierror as e:
            LOG.info("Unable to get host name for %s: %s", url, e)
            return False
        for net in self.private_networks:
            if ip in net:
                return False
        return True

    @staticmethod
    def phash():  # pragma: no cover
        return ""


class MapserverproxyRoutePredicate:
    """ Serve as a custom route predicate function for mapserverproxy.
    If the hide_capabilities setting is set and is true then we want to
    return 404s on GetCapabilities requests."""

    def __init__(self, val, config):
        pass

    def __call__(self, context, request):
        hide_capabilities = request.registry.settings.get("hide_capabilities")
        if not hide_capabilities:
            return True
        params = dict(
            (k.lower(), v.lower()) for k, v in request.params.items()
        )
        return "request" not in params or params["request"] != "getcapabilities"

    @staticmethod
    def phash():
        return ""


def add_cors_route(config, pattern, service):
    """
    Add the OPTIONS route and view need for services supporting CORS.
    """
    def view(request):  # pragma: no cover
        from c2cgeoportal_geoportal.lib.caching import set_common_headers, NO_CACHE
        return set_common_headers(request, service, NO_CACHE)

    name = pattern + "_options"
    config.add_route(name, pattern, request_method="OPTIONS")
    config.add_view(view, route_name=name)


def error_handler(http_exception, request):  # pragma: no cover
    """
    View callable for handling all the exceptions that are not already handled.
    """
    LOG.warning("%s returned status code %s", request.url, http_exception.status_code)
    return caching.set_common_headers(
        request, "error", caching.NO_CACHE, http_exception, vary=True
    )


def call_hook(settings, name, *args, **kwargs):
    hooks = settings.get("hooks", {})
    hook = hooks.get(name)
    if hook is None:
        return
    parts = hook.split(".")
    module = importlib.import_module(".".join(parts[0:-1]))
    function_ = getattr(module, parts[-1])
    function_(*args, **kwargs)


def includeme(config: pyramid.config.Configurator):
    """
    This function returns a Pyramid WSGI application.
    """

    settings = config.get_settings()

    config.include("c2cgeoportal_commons")

    call_hook(settings, "after_settings", settings)

    get_user_from_request = create_get_user_from_request(settings)
    config.add_request_method(get_user_from_request, name="user", property=True)
    config.add_request_method(get_user_from_request, name="get_user")

    # Configure 'locale' dir as the translation dir for c2cgeoportal app
    config.add_translation_dirs("c2cgeoportal_geoportal:locale/")

    config.include('c2cwsgiutils.pyramid.includeme')
    health_check = HealthCheck(config)

    # Initialise DBSessions
    init_dbsessions(settings, config, health_check)

    # Initialize the dbreflection module
    dbreflection.init()

    checker.init(config, health_check)
    check_collector.init(config, health_check)

    # dogpile.cache configuration
    if 'cache' in settings:
        caching.init_region(settings['cache'])
        from c2cgeoportal_commons.models.main import InvalidateCacheEvent

        @zope.event.classhandler.handler(InvalidateCacheEvent)
        def handle(event: InvalidateCacheEvent):  # pylint: disable=unused-variable
            del event
            caching.invalidate_region()

    # Register a tween to get back the cache buster path.
    if 'cache_path' not in config.get_settings():
        config.get_settings()['cache_path'] = ['static']
    config.add_tween("c2cgeoportal_geoportal.lib.cacheversion.CachebusterTween")
    config.add_tween("c2cgeoportal_geoportal.lib.headers.HeadersTween")

    # Bind the mako renderer to other file extensions
    add_mako_renderer(config, ".html")
    add_mako_renderer(config, ".js")

    package = config.get_settings()["package"]

    # Add the "geojson" renderer
    config.add_renderer("geojson", GeoJSON())

    # Add decimal json renderer
    config.add_renderer("decimaljson", DecimalJSON())

    # Add the "xsd" renderer
    config.add_renderer("xsd", XSD(
        include_foreign_keys=True
    ))

    # Add the set_user_validator directive, and set a default user validator
    config.add_directive("set_user_validator", set_user_validator)
    config.set_user_validator(default_user_validator)

    config.add_route('dynamic', '/dynamic.json', request_method="GET")
    if settings.get("ogcproxy_enable", False):  # pragma: no cover
        # Add an OGCProxy view
        config.add_route_predicate("ogc_server", OgcproxyRoutePredicate)
        config.add_route(
            "ogcproxy", "/ogcproxy",
            ogc_server=True
        )
        config.add_view("papyrus_ogcproxy.views:ogcproxy", route_name="ogcproxy")

    # Add routes to the mapserver proxy
    config.add_route_predicate("mapserverproxy", MapserverproxyRoutePredicate)
    config.add_route(
        "mapserverproxy", "/mapserv_proxy",
        mapserverproxy=True, pregenerator=C2CPregenerator(role=True),
    )

    # Add route to the tinyows proxy
    config.add_route(
        "tinyowsproxy", "/tinyows_proxy",
        pregenerator=C2CPregenerator(role=True),
    )

    # Add routes to the entry view class
    config.add_route("base", "/", static=True)
    config.add_route("loginform", "/login.html", request_method="GET")
    add_cors_route(config, "/login", "login")
    config.add_route("login", "/login", request_method="POST")
    add_cors_route(config, "/logout", "login")
    config.add_route("logout", "/logout", request_method="GET")
    add_cors_route(config, "/loginchange", "login")
    config.add_route("loginchange", "/loginchange", request_method="POST")
    add_cors_route(config, "/loginresetpassword", "login")
    config.add_route("loginresetpassword", "/loginresetpassword", request_method="POST")
    add_cors_route(config, "/loginuser", "login")
    config.add_route("loginuser", "/loginuser", request_method="GET")
    config.add_route("testi18n", "/testi18n.html", request_method="GET")

    config.add_renderer(".map", AssetRendererFactory)
    config.add_renderer(".css", AssetRendererFactory)
    config.add_renderer(".ico", AssetRendererFactory)
    config.add_route("apijs", "/api.js", request_method="GET")
    config.add_route("oldxapijs", "/xapi.js", request_method="GET")
    config.add_route("oldapihelp", "/oldapihelp.html", request_method="GET")
    config.add_route("oldxapihelp", "/oldxapihelp.html", request_method="GET")

    # Cannot be at the header to do not load the model too early
    from c2cgeoportal_geoportal.views.entry import Entry

    def add_api_route(name: str, attr: str, path: str, renderer: str):
        config.add_route(name, path, request_method="GET")
        config.add_view(
            Entry, attr=attr, route_name=name,
            renderer='{}_geoportal:static-ngeo/{}'.format(package, renderer)
        )
    add_api_route('favicon', 'favicon', '/favicon.ico', 'js/apps/image/favicon.ico')
    add_api_route('apijsmap', 'apijsmap', '/api.js.map', "build/api.js.map")
    add_api_route('apicss', 'apicss', '/api.css', "build/api.css")
    add_api_route('apihelp', 'apihelp', '/apihelp/index.html', "api/apihelp/index.html")
    config.add_route(
        "themes", "/themes",
        request_method="GET",
        pregenerator=C2CPregenerator(role=True),
    )

    config.add_route("invalidate", "/invalidate", request_method="GET")

    # Print proxy routes
    config.add_route("printproxy", "/printproxy", request_method="HEAD")
    add_cors_route(config, "/printproxy/*all", "print")
    config.add_route(
        "printproxy_capabilities", "/printproxy/capabilities.json",
        request_method="GET", pregenerator=C2CPregenerator(role=True),
    )
    config.add_route(
        "printproxy_report_create", "/printproxy/report.{format}",
        request_method="POST", header=JSON_CONTENT_TYPE
    )
    config.add_route(
        "printproxy_status", "/printproxy/status/{ref}.json",
        request_method="GET"
    )
    config.add_route(
        "printproxy_cancel", "/printproxy/cancel/{ref}",
        request_method="DELETE"
    )
    config.add_route(
        "printproxy_report_get", "/printproxy/report/{ref}",
        request_method="GET"
    )

    # Full-text search routes
    add_cors_route(config, "/search", "fulltextsearch")
    config.add_route("fulltextsearch", "/search")

    # Access to raster data
    add_cors_route(config, "/raster", "raster")
    config.add_route("raster", "/raster", request_method="GET")

    add_cors_route(config, "/profile.{ext}", "profile")
    config.add_route("profile.csv", "/profile.csv", request_method="POST")
    config.add_route("profile.json", "/profile.json", request_method="POST")

    # Shortener
    add_cors_route(config, "/short/create", "shortener")
    config.add_route("shortener_create", "/short/create", request_method="POST")
    config.add_route("shortener_get", "/s/{ref}", request_method="GET")

    # Geometry processing
    config.add_route("difference", "/difference", request_method="POST")

    # PDF report tool
    config.add_route("pdfreport", "/pdfreport/{layername}/{ids}", request_method="GET")

    # Add routes for the "layers" web service
    add_cors_route(config, "/layers/*all", "layers")
    config.add_route(
        "layers_count", "/layers/{layer_id:\\d+}/count",
        request_method="GET"
    )
    config.add_route(
        "layers_metadata", "/layers/{layer_id:\\d+}/md.xsd",
        request_method="GET",
        pregenerator=C2CPregenerator(role=True),
    )
    config.add_route(
        "layers_read_many",
        "/layers/{layer_id:\\d+,?(\\d+,)*\\d*$}",
        request_method="GET")  # supports URLs like /layers/1,2,3
    config.add_route(
        "layers_read_one", "/layers/{layer_id:\\d+}/{feature_id}",
        request_method="GET")
    config.add_route(
        "layers_create", "/layers/{layer_id:\\d+}",
        request_method="POST", header=GEOJSON_CONTENT_TYPE)
    config.add_route(
        "layers_update", "/layers/{layer_id:\\d+}/{feature_id}",
        request_method="PUT", header=GEOJSON_CONTENT_TYPE)
    config.add_route(
        "layers_delete", "/layers/{layer_id:\\d+}/{feature_id}",
        request_method="DELETE")
    config.add_route(
        "layers_enumerate_attribute_values",
        "/layers/{layer_name}/values/{field_name}",
        request_method="GET",
        pregenerator=C2CPregenerator(),
    )
    # There is no view corresponding to that route, it is to be used from
    # mako templates to get the root of the "layers" web service
    config.add_route("layers_root", "/layers", request_method="HEAD")

    # Resource proxy (load external url, useful when loading non https content)
    config.add_route("resourceproxy", "/resourceproxy", request_method="GET")

    # Dev
    config.add_route("dev", "/dev/*path", request_method="GET")

    # Used memory in caches
    add_cors_route(config, "/memory", "memory")
    config.add_route("memory", "/memory", request_method="GET")

    # Scan view decorator for adding routes
    config.scan(ignore=[
        "c2cgeoportal_geoportal.lib",
        "c2cgeoportal_geoportal.scaffolds",
        "c2cgeoportal_geoportal.scripts"
    ])

    if "subdomains" in settings:  # pragma: no cover
        config.registry.registerUtility(
            MultiDomainStaticURLInfo(), IStaticURLInfo)

    add_admin_interface(config)

    # Add the project static view with cache buster
    from c2cgeoportal_geoportal.lib.cacheversion import version_cache_buster
    config.add_static_view(
        name="static",
        path="{}_geoportal:static".format(package),
        cache_max_age=int(config.get_settings()["default_max_age"]),
    )
    config.add_cache_buster("{}_geoportal:static".format(package), version_cache_buster)

    # Add the project static view without cache buster
    config.add_static_view(
        name="static-ngeo",
        path="{}_geoportal:static-ngeo".format(package),
        cache_max_age=int(config.get_settings()["default_max_age"]),
    )

    # Handles the other HTTP errors raised by the views. Without that,
    # the client receives a status=200 without content.
    config.add_view(error_handler, context=HTTPException)

    c2cwsgiutils.index.additional_title = '<div class="row"><div class="col-lg-3"><h2>GeoMapFish</h2>' \
        '</div><div class="col-lg">'
    c2cwsgiutils.index.additional_auth.extend([
        '<a href="../tiles/admin/">TileCloud chain admin</a><br>',
        '<a href="../invalidate">Invalidate the cache</a><br>',
        '<a href="../memory">Memory status</a><br>',
    ])
    if config.get_settings().get('enable_admin_interface', False):
        c2cwsgiutils.index.additional_noauth.append('<a href="../admin/">Admin</a><br>')

    c2cwsgiutils.index.additional_noauth.append(
        '</div></div><div class="row"><div class="col-lg-3"><h3>Interfaces</h3></div><div class="col-lg">'
    )
    c2cwsgiutils.index.additional_noauth.append('<a href="../">Default</a><br>')
    for interface in config.get_settings().get("interfaces", []):
        if interface != config.get_settings().get("default_interface"):
            c2cwsgiutils.index.additional_noauth.append(
                '<a href="../{interface}">{interface}</a><br>'.format(
                    interface=interface
                )
            )
    c2cwsgiutils.index.additional_noauth.append('<a href="../apihelp/index.html">API help</a><br>')
    c2cwsgiutils.index.additional_noauth.append('<a href="../oldapihelp.html">Old API help</a><br>')
    c2cwsgiutils.index.additional_noauth.append('<a href="../oldxapihelp.html">Old extended API help</a><br>')
    c2cwsgiutils.index.additional_noauth.append('</div></div><hr>')


def init_dbsessions(settings: dict, config: Configurator, health_check: HealthCheck = None) -> None:
    db_chooser = settings.get('db_chooser', {})
    master_paths = [re.compile(i.replace('//', '/')) for i in db_chooser.get('master', [])]
    slave_paths = [re.compile(i.replace('//', '/')) for i in db_chooser.get('slave', [])]

    slave_prefix = 'sqlalchemy_slave' if 'sqlalchemy_slave.url' in settings else None

    c2cgeoportal_commons.models.DBSession, rw_bind, _ = c2cwsgiutils.db.setup_session(
        config, 'sqlalchemy', slave_prefix, force_master=master_paths, force_slave=slave_paths)
    c2cgeoportal_commons.models.Base.metadata.bind = rw_bind
    c2cgeoportal_commons.models.DBSessions['dbsession'] = c2cgeoportal_commons.models.DBSession

    for dbsession_name, dbsession_config in settings.get('dbsessions', {}).items():  # pragma: nocover
        c2cgeoportal_commons.models.DBSessions[dbsession_name] = \
            c2cwsgiutils.db.create_session(config, dbsession_name, **dbsession_config)

    c2cgeoportal_commons.models.Base.metadata.clear()
    from c2cgeoportal_commons.models import main

    if health_check is not None:
        for name, session in c2cgeoportal_commons.models.DBSessions.items():
            if name == 'dbsession':
                health_check.add_db_session_check(
                    session, at_least_one_model=main.Theme, level=1
                )
                alembic_ini = os.path.join(os.path.abspath(os.path.curdir), 'alembic.ini')
                if os.path.exists(alembic_ini):
                    health_check.add_alembic_check(
                        session, alembic_ini_path=alembic_ini, name='main',
                        version_schema=settings['schema'], level=1
                    )
                    health_check.add_alembic_check(
                        session, alembic_ini_path=alembic_ini, name='static',
                        version_schema=settings['schema_static'], level=1
                    )
            else:  # pragma: no cover
                def check(session_: Session) -> None:
                    session_.execute('SELECT 1')
                health_check.add_db_session_check(session, query_cb=check, level=2)



# -*- coding: utf-8 -*-

# Copyright (c) 2013-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

# pylint: disable=missing-docstring,attribute-defined-outside-init,protected-access


from unittest import TestCase
from pyramid import testing
from c2c.template.config import config


def setup_module():  # noqa
    from c2cgeoportal_geoportal import caching
    config._config = {
        'schema': 'main',
        'schema_static': 'main_static',
        'srid': 21781,
    }
    caching.init_region({
        "backend": "dogpile.cache.null",
    })


class TestEntryView(TestCase):

    def setup_method(self, _):
        testing.setUp(
            settings={
                "default_locale_name": "fr",
                "default_max_age": 1000,
            }
        )

    def teardown_method(self, _):
        testing.tearDown()

    def test_decimal_json(self):
        from decimal import Decimal
        from pyramid.testing import DummyRequest
        from c2cgeoportal_geoportal import DecimalJSON
        renderer = DecimalJSON()(None)
        request = DummyRequest()
        request.user = None
        system = {"request": request}

        self.assertEqual(
            renderer({"a": Decimal("3.3")}, system),
            '{"a": 3.3}'
        )
        self.assertEqual(request.response.content_type, "application/json")

    def test__get_child_layers_info_with_scalehint(self):
        import math
        from pyramid.testing import DummyRequest
        from c2cgeoportal_geoportal.views.entry import Entry

        request = DummyRequest()
        request.user = None
        entry = Entry(request)

        class Layer:
            pass

        child_layer_1 = Layer()
        child_layer_1.name = "layer_1"
        child_layer_1.scaleHint = {
            "min": 1 * math.sqrt(2),
            "max": 2 * math.sqrt(2)
        }
        child_layer_1.layers = []

        child_layer_2 = Layer()
        child_layer_2.name = "layer_2"
        child_layer_2.scaleHint = {
            "min": 3 * math.sqrt(2),
            "max": 4 * math.sqrt(2)
        }
        child_layer_2.layers = []

        layer = Layer()
        layer.layers = [child_layer_1, child_layer_2]

        child_layers_info = entry._get_child_layers_info_1(layer)

        expected = [{
            "name": "layer_1",
            "minResolutionHint": 1.0,
            "maxResolutionHint": 2.0
        }, {
            "name": "layer_2",
            "minResolutionHint": 3.0,
            "maxResolutionHint": 4.0
        }]
        self.assertEqual(child_layers_info, expected)

    def test_login(self):
        from pyramid.testing import DummyRequest
        from c2cgeoportal_geoportal.views.entry import Entry

        request = DummyRequest()
        request.is_valid_referer = True
        request.user = None
        entry = Entry(request)

        request.path = "/for_test"
        expected = {
            "lang": "fr",
            "came_from": "/for_test",
        }
        self.assertEqual(entry.loginform403(), expected)

        request.params = {
            "came_from": "/for_a_second_test",
        }
        entry = Entry(request)
        expected = {
            "lang": "fr",
            "came_from": "/for_a_second_test",
        }
        self.assertEqual(entry.loginform(), expected)

        entry = Entry(request)
        request.params = {}
        expected = {
            "lang": "fr",
            "came_from": "/",
        }
        self.assertEqual(entry.loginform(), expected)

        request.registry.settings = {
            "functionalities": {
                "anonymous": {
                    "func": ["anon"],
                    "toto": "titi"
                },
                "registered": {
                    "func": ["reg"]
                },
                "available_in_templates": ["func"]
            },
            "admin_interface": {
                "available_functionalities": [{"name": "func"}]
            }
        }
        entry = Entry(request)
        expected = {
            "functionalities": {
                "func": ["anon"]
            }
        }
        self.assertEqual(entry.loginuser(), expected)

        class G:
            id = 123

            def __init__(self, name, functionalities):
                self.name = name
                self.functionalities = functionalities

        class U:
            username = "__test_user"
            is_password_changed = True
            email = "info@example.com"

            def __init__(self, role="__test_role", functionalities=None):
                if functionalities is None:
                    functionalities = []
                self.roles = [G(role, functionalities)]

        request.user = U()
        entry = Entry(request)
        expected = {
            "success": True,
            "username": "__test_user",
            "email": "info@example.com",
            "is_password_changed": True,
            "roles": [{
                "name": "__test_role",
                "id": 123,
            }],
            "functionalities": {
                "func": ["reg"]
            }
        }
        self.assertEqual(entry.loginuser(), expected)

        class F:
            name = "func"
            value = "user"

        request.user = U("__test_role2", [F()])
        entry = Entry(request)
        expected = {
            "success": True,
            "username": "__test_user",
            'email': 'info@example.com',
            "is_password_changed": True,
            "roles": [{
                "name": "__test_role2",
                "id": 123,
            }],
            "functionalities": {
                "func": ["user"]
            }
        }
        self.assertEqual(entry.loginuser(), expected)

    def test__get_child_layers_info_without_scalehint(self):
        from pyramid.testing import DummyRequest
        from c2cgeoportal_geoportal.views.entry import Entry

        request = DummyRequest()
        request.user = None
        entry = Entry(request)

        class Layer:
            pass

        child_layer_1 = Layer()
        child_layer_1.name = "layer_1"
        child_layer_1.scaleHint = None
        child_layer_1.layers = []

        child_layer_2 = Layer()
        child_layer_2.name = "layer_2"
        child_layer_2.scaleHint = None
        child_layer_2.layers = []

        layer = Layer()
        layer.layers = [child_layer_1, child_layer_2]

        child_layers_info = entry._get_child_layers_info_1(layer)

        expected = [{
            "name": "layer_1",
            "minResolutionHint": 0.0,
            "maxResolutionHint": 999999999.0,
        }, {
            "name": "layer_2",
            "minResolutionHint": 0.0,
            "maxResolutionHint": 999999999.0,
        }]
        self.assertEqual(child_layers_info, expected)



# -*- coding: utf-8 -*-

# Copyright (c) 2018-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import urllib.parse
from c2cgeoportal_commons import models
from c2cgeoportal_commons.models import main
from c2cgeoportal_geoportal.lib.cacheversion import get_cache_version
from c2cgeoportal_geoportal.lib.caching import set_common_headers, NO_CACHE
from pyramid.view import view_config
from sqlalchemy import func


class DynamicView:

    def __init__(self, request):
        self.request = request
        self.settings = request.registry.settings
        self.interfaces_config = self.settings['interfaces_config']
        self.default = self.interfaces_config.get('default', {})

    def get(self, value, interface):
        result = dict(self.default.get(value, {}))
        result.update(self.interfaces_config.get(interface, {}).get(value, {}))
        return result

    @view_config(route_name='dynamic', renderer='fast_json')
    def dynamic(self):
        interface_name = self.request.params.get('interface')
        if interface_name not in self.settings.get('interfaces'):
            interface_name = self.settings.get('default_interface')
        interface_config = self.interfaces_config[interface_name]

        dynamic = {
            'interface': interface_name,
            'cache_version': get_cache_version(),
            'lang_urls': {
                lang: self.request.static_url('{package}_geoportal:static-ngeo/build/{lang}.json'.format(
                    package=self.request.registry.settings["package"], lang=lang
                ))
                for lang in self.request.registry.settings["available_locale_names"]
            },
            'fulltextsearch_groups': [
                group[0] for group in models.DBSession.query(
                    func.distinct(main.FullTextSearch.layer_name)
                ).filter(main.FullTextSearch.layer_name.isnot(None)).all()
            ],
        }

        constants = {name: value for name, value in self.get('constants', interface_name).items()}
        constants.update({
            name: dynamic[value]
            for name, value in self.get('dynamic_constants', interface_name).items()
        })
        constants.update({
            name: self.request.static_url(static_['name']) + static_.get('append', '')
            for name, static_ in self.get('static', interface_name).items()
        })

        routes = dict(currentInterfaceUrl={'name': interface_name})
        routes.update(self.get('routes', interface_name))
        for constant, config in routes.items():
            params = {}
            params.update(config.get('params', {}))
            for name, dyn in config.get('dynamic_params', {}).items():
                params[name] = dynamic[dyn]
            constants[constant] = self.request.route_url(config['name'], _query=params)

        do_redirect = False
        url = None
        if 'redirect_interface' in interface_config:
            no_redirect_query = {
                'no_redirect': 't'
            }
            if 'query' in self.request.params:
                query = urllib.parse.parse_qs(self.request.params['query'][1:])
                no_redirect_query.update(query)
            else:
                query = {}
            if 'themes' in self.request.matchdict:
                no_redirect_url = self.request.route_url(
                    interface_config['redirect_interface'] + 'theme',
                    themes=self.request.matchdict['themes'],
                    _query=no_redirect_query
                )
                url = self.request.route_url(
                    interface_config['redirect_interface'] + 'theme',
                    themes=self.request.matchdict['themes'],
                    _query=query
                )
            else:
                no_redirect_url = self.request.route_url(
                    interface_config['redirect_interface'],
                    _query=no_redirect_query
                )
                url = self.request.route_url(
                    interface_config['redirect_interface'],
                    _query=query
                )

            if 'no_redirect' in query:
                constants['redirectUrl'] = ''
            else:
                if interface_config.get('do_redirect', False):
                    do_redirect = True
                else:
                    constants['redirectUrl'] = no_redirect_url

        set_common_headers(self.request, "dynamic", NO_CACHE)
        return {
            'constants': constants,
            'doRedirect': do_redirect,
            'redirectUrl': url,
        }



# -*- coding: utf-8 -*-

# Copyright (c) 2013-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

# pylint: disable=missing-docstring,attribute-defined-outside-init,protected-access


from urllib.parse import urlparse, parse_qs
from unittest import TestCase

from c2cgeoportal_geoportal.lib import add_url_params


class TestLib(TestCase):

    def test_add_url_params_encode1(self):
        presult = urlparse(add_url_params(
            "http://example.com/toto",
            {"à": "é"}
        ))
        self.assertEqual(presult.scheme, "http")
        self.assertEqual(presult.netloc, "example.com")
        self.assertEqual(presult.path, "/toto")
        self.assertEqual(presult.params, "")
        self.assertEqual(presult.fragment, "")
        self.assertEqual(parse_qs(presult.query), {
            "à": ["é"],
        })

    def test_add_url_params_encode2(self):
        presult = urlparse(add_url_params(
            "http://example.com/toto?à=é",
            {"1": "2"}
        ))
        self.assertEqual(presult.scheme, "http")
        self.assertEqual(presult.netloc, "example.com")
        self.assertEqual(presult.path, "/toto")
        self.assertEqual(presult.params, "")
        self.assertEqual(presult.fragment, "")
        self.assertEqual(parse_qs(presult.query), {
            "à": ["é"],
            "1": ["2"],
        })

    def test_add_url_params_encode3(self):
        presult = urlparse(add_url_params(
            "http://example.com/toto?%C3%A0=%C3%A9",
            {"1": "2"}
        ))
        self.assertEqual(presult.scheme, "http")
        self.assertEqual(presult.netloc, "example.com")
        self.assertEqual(presult.path, "/toto")
        self.assertEqual(presult.params, "")
        self.assertEqual(presult.fragment, "")
        self.assertEqual(parse_qs(presult.query), {
            "à": ["é"],
            "1": ["2"],
        })

    def test_add_url_params_port(self):
        presult = urlparse(add_url_params(
            "http://example.com:8480/toto",
            {"1": "2"}
        ))
        self.assertEqual(presult.scheme, "http")
        self.assertEqual(presult.netloc, "example.com:8480")
        self.assertEqual(presult.path, "/toto")
        self.assertEqual(presult.params, "")
        self.assertEqual(presult.fragment, "")
        self.assertEqual(parse_qs(presult.query), {
            "1": ["2"],
        })

    def test_add_url_params_noparam(self):
        self.assertEqual(add_url_params(
            "http://example.com/", {}
        ), "http://example.com/")



# -*- coding: utf-8 -*-

# Copyright (c) 2013-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

# pylint: disable=missing-docstring,attribute-defined-outside-init,protected-access


from unittest import TestCase
from tests import create_dummy_request


class TestGetURL(TestCase):

    def test_get_url2(self):
        from c2cgeoportal_geoportal.lib import get_url2

        request = create_dummy_request({
            "package": "my_project",
            "servers": {
                "srv": "https://example.com/test",
                "srv_alt": "https://example.com/test/",
                "full_url": "https://example.com/test.xml",
            },
        })

        def static_url(path, **kwargs):
            del kwargs  # Unused
            return "http://server.org/" + path
        request.static_url = static_url

        self.assertEqual(get_url2("test", "static://pr:st/icon.png", request, set()), "http://server.org/pr:st/icon.png")
        self.assertEqual(get_url2("test", "static:///icon.png", request, set()), "http://server.org/my_project_geoportal:static/icon.png")
        self.assertEqual(get_url2("test", "config://srv/icon.png", request, set()), "https://example.com/test/icon.png")
        self.assertEqual(get_url2("test", "config://srv/", request, set()), "https://example.com/test/")
        self.assertEqual(get_url2("test", "config://srv", request, set()), "https://example.com/test")
        self.assertEqual(get_url2("test", "config://srv/icon.png?test=aaa", request, set()), "https://example.com/test/icon.png?test=aaa")
        self.assertEqual(get_url2("test", "config://srv_alt/icon.png", request, set()), "https://example.com/test/icon.png")
        self.assertEqual(get_url2("test", "config://full_url", request, set()), "https://example.com/test.xml")
        self.assertEqual(get_url2("test", "http://example.com/icon.png", request, set()), "http://example.com/icon.png")
        self.assertEqual(get_url2("test", "https://example.com/icon.png", request, set()), "https://example.com/icon.png")
        errors = set()
        self.assertEqual(get_url2("test", "config://srv2/icon.png", request, errors=errors), None)
        self.assertEqual(errors, set(["test: The server 'srv2' (config://srv2/icon.png) is not found in the config: [srv, srv_alt, full_url]"]))



import os
from typing import Dict
from plaster_pastedeploy import Loader as BaseLoader

from c2c.template.config import config as configuration


class Loader(BaseLoader):

    def _get_defaults(self, defaults: Dict[str, str] = None) -> Dict[str, str]:
        d = {
            "VISIBLE_ENTRY_POINT": "cli",
            "LOG_LEVEL": "INFO",
            "C2CGEOPORTAL_LOG_LEVEL": "WARN",
            "GUNICORN_LOG_LEVEL": "INFO",
            "GUNICORN_ACCESS_LOG_LEVEL": "INFO",
            "SQL_LOG_LEVEL": "INFO",
            "OTHER_LOG_LEVEL": "INFO",
            "LOG_HOST": "localhost",
            "LOG_PORT": "0",
        }  # type: Dict[str, str]
        d.update(os.environ)
        if defaults:
            d.update(defaults)
        return super()._get_defaults(d)

    def get_wsgi_app_settings(self, name: str = None, defaults: Dict[str, str] = None) -> Dict:
        settings = super().get_wsgi_app_settings(name, defaults)
        configuration.init(settings.get('app.cfg'))
        settings.update(configuration.get_config())
        return settings

    def __repr__(self) -> str:
        return 'c2cgeoportal_geoportal.lib.loader.Loader(uri="{0}")'.format(self.uri)



# -*- coding: utf-8 -*-

# Copyright (c) 2013-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

# pylint: disable=missing-docstring,attribute-defined-outside-init,protected-access


from unittest import TestCase
import datetime
import isodate


class TestExtent(TestCase):
    def test_parse_values(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import parse_extent, TimeExtentValue
        extent = parse_extent(["2005", "2006"], "2005")
        self.assertTrue(isinstance(extent, TimeExtentValue))

    def test_parse_interval(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import parse_extent, TimeExtentInterval
        extent = parse_extent(["2000/2005/P1Y"], "2002")
        self.assertTrue(isinstance(extent, TimeExtentInterval))

    def test_unsupported_format(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import parse_extent
        self.assertRaises(ValueError, parse_extent, ["2000/2010"], "2002")
        self.assertRaises(ValueError, parse_extent, [], "2002")

    def test_merge_values(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import parse_extent, TimeExtentValue
        e1 = parse_extent(["2000", "2005"], "2000/2005")
        e2 = parse_extent(["2001", "2003"], "2001/2003")
        self.assertTrue(isinstance(e1, TimeExtentValue))
        self.assertTrue(isinstance(e2, TimeExtentValue))
        e1.merge(e2)
        d = e1.to_dict()
        self.assertEqual(d, {
            "minValue": "2000-01-01T00:00:00Z",
            "maxValue": "2005-01-01T00:00:00Z",
            "resolution": "year",
            "values": [
                "2000-01-01T00:00:00Z",
                "2001-01-01T00:00:00Z",
                "2003-01-01T00:00:00Z",
                "2005-01-01T00:00:00Z"
            ],
            "minDefValue": "2000-01-01T00:00:00Z",
            "maxDefValue": "2005-01-01T00:00:00Z",
        })

    def test_merge_interval(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import parse_extent, TimeExtentInterval
        e1 = parse_extent(["2000/2005/P1Y"], "2000/2005")
        e2 = parse_extent(["2006/2010/P1Y"], "2006/2010")
        self.assertTrue(isinstance(e1, TimeExtentInterval))
        self.assertTrue(isinstance(e2, TimeExtentInterval))
        e1.merge(e2)
        d = e1.to_dict()
        self.assertEqual(d, {
            "minValue": "2000-01-01T00:00:00Z",
            "maxValue": "2010-01-01T00:00:00Z",
            "resolution": "year",
            "interval": (1, 0, 0, 0),
            "minDefValue": "2000-01-01T00:00:00Z",
            "maxDefValue": "2010-01-01T00:00:00Z",
        })

    def test_merge_value_interval(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import parse_extent
        tev = parse_extent(["2000", "2001"], "2000")
        tei = parse_extent(["2000/2010/P1Y"], "2002")
        self.assertRaises(ValueError, tev.merge, tei)
        self.assertRaises(ValueError, tei.merge, tev)

    def test_merge_different_intervals(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import parse_extent
        e1 = parse_extent(["2000/2005/P1Y"], "2002")
        e2 = parse_extent(["2006/2010/P1M"], "2002")
        self.assertRaises(ValueError, e1.merge, e2)


class TestParseDate(TestCase):
    def test_parse_date_year(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_date
        date = _parse_date("2010")
        self.assertEqual("year", date[0])
        self.assertEqual(
            datetime.datetime(2010, 0o1, 0o1, tzinfo=isodate.UTC),
            date[1]
        )

    def test_parse_date_month(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_date
        date = _parse_date("2010-02")
        self.assertEqual("month", date[0])
        self.assertEqual(
            datetime.datetime(2010, 0o2, 0o1, tzinfo=isodate.UTC),
            date[1]
        )

    def test_parse_date(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_date
        date = _parse_date("2010-02-03")
        self.assertEqual("day", date[0])
        self.assertEqual(
            datetime.datetime(2010, 0o2, 0o3, tzinfo=isodate.UTC),
            date[1]
        )

    def test_parse_datetime(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_date
        date = _parse_date("2010-02-03T12:34")
        self.assertEqual("second", date[0])
        self.assertEqual(
            datetime.datetime(2010, 0o2, 0o3, 12, 34, tzinfo=isodate.UTC),
            date[1]
        )

    def test_parse_datetime_tz(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_date
        date = _parse_date("2010-02-03T12:34Z")
        self.assertEqual("second", date[0])
        self.assertEqual(
            datetime.datetime(2010, 0o2, 0o3, 12, 34, tzinfo=isodate.UTC),
            date[1]
        )

    def test_unsupported_format(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_date
        self.assertRaises(ValueError, _parse_date, "2010-02-03 12:34")


class TestFormat(TestCase):
    def test_format(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _format_date
        dt = datetime.datetime(2010, 0o2, 0o1, 00, 00)
        self.assertEqual("2010-02-01T00:00:00Z", _format_date(dt))

    def test_format_tz(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_date, _format_date
        dt = _parse_date("2010-02-03T12:34:00+01:00")
        self.assertEqual("2010-02-03T12:34:00+01:00", _format_date(dt[1]))


class TestParseDuration(TestCase):
    def test_year(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_duration
        self.assertEqual((2, 0, 0, 0), _parse_duration("P2Y"))

    def test_month(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_duration
        self.assertEqual((0, 2, 0, 0), _parse_duration("P2M"))

    def test_day(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_duration
        self.assertEqual((0, 0, 2, 0), _parse_duration("P2D"))

    def test_hour(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_duration
        self.assertEqual((0, 0, 0, 3600), _parse_duration("PT1H"))

    def test_minute(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_duration
        self.assertEqual((0, 0, 0, 600), _parse_duration("PT10M"))

    def test_second(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_duration
        self.assertEqual((0, 0, 0, 10), _parse_duration("PT10S"))

    def test_invalid(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import _parse_duration
        from isodate import ISO8601Error
        self.assertRaises(ISO8601Error, _parse_duration, "10S")


class TestTimeInformation(TestCase):
    def test_merge_modes(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import TimeInformation
        ti = TimeInformation()
        self.assertFalse(ti.has_time())
        self.assertTrue(ti.to_dict() is None)
        ti.merge_mode("single")
        self.assertEqual(ti.mode, "single")
        ti.merge_mode("single")
        self.assertEqual(ti.mode, "single")

    def test_merge_different_modes(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import TimeInformation
        ti = TimeInformation()
        ti.merge_mode("single")
        self.assertRaises(ValueError, ti.merge_mode, "range")

    def test_merge_different_widgets(self):
        from c2cgeoportal_geoportal.lib.wmstparsing import TimeInformation
        ti = TimeInformation()
        ti.merge_widget("single")
        self.assertRaises(ValueError, ti.merge_widget, "datepicker")



# -*- coding: utf-8 -*-

# Copyright (c) 2017-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

import argparse
import time
import json
import binascii
import c2c.template
from Crypto.Cipher import AES


def create_token(aeskey, user, password, valid):
    auth = {
        "u": user,
        "p": password,
        "t": int(time.time()) + valid * 3600 * 24,
    }

    if aeskey is None:
        print("urllogin is not configured")
        exit(1)
    cipher = AES.new(aeskey.encode("ascii"), AES.MODE_EAX)
    data = json.dumps(auth)
    mod_len = len(data) % 16
    if mod_len != 0:
        data += "".join([" " for i in range(16 - mod_len)])
    ciphertext, tag = cipher.encrypt_and_digest(data.encode("utf-8"))
    return binascii.hexlify(cipher.nonce + tag + ciphertext).decode("ascii")


def main():
    parser = argparse.ArgumentParser(description="Generate an auth token")
    parser.add_argument("user", help="The username")
    parser.add_argument("password", help="The password")
    parser.add_argument("valid", type=int, default=1, nargs='?', help="Is valid for, in days")

    args = parser.parse_args()
    config = c2c.template.get_config("config.yaml")
    urllogin = config.get('urllogin', {})
    aeskey = urllogin.get("aes_key")
    auth_enc = create_token(aeskey, args.user, args.password, args.valid)

    print(("Use: auth={}".format(auth_enc)))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2014-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import atexit
import os
import re
import sys
import argparse
import yaml
import shutil
import pkg_resources
import subprocess
import filecmp
from subprocess import check_call, call, check_output
from argparse import ArgumentParser
from alembic.config import Config
from alembic import command
import requests

from c2cgeoportal_geoportal.lib.bashcolor import colorize, GREEN, YELLOW, RED

REQUIRED_TEMPLATE_KEYS = ["package", "srid", "extent"]
TEMPLATE_EXAMPLE = {
    "package": "${package}",
    "srid": "${srid}",
    "extent": "489246, 78873, 837119, 296543",
}
DIFF_NOTICE = "You should apply the changes shown in the diff file on `CONST_create_template/<file>` " \
    "on your project's `<file>`.\n" \
    "Some advice to be more efficient: if the changes on a file concern a file that you never customize, " \
    "you can simply copy the new file from `CONST_create_template` " \
    "(`cp CONST_create_template/<file> <file>`)." \
    "You can furthermore add this file to the `unmanaged_files` section of the `project.yaml.mako` file, " \
    "to avoid its contents appearing in the diff file for the next upgrade."


def main():
    """
    tool used to do the application upgrade
    """

    parser = _fill_arguments()
    options = parser.parse_args()
    if options.force_docker:
        options.nondocker = False
    if options.new_makefile is None:
        options.new_makefile = options.makefile

    print("Starting the upgrade with options:")
    if options.windows:
        print("- windows")
    if options.nondocker:
        print("- nondocker")
    print("- git_remote=" + options.git_remote)
    if options.use_makefile:
        print("- use_makefile")
    print("- makefile=" + options.makefile)
    print("- new_makefile=" + options.new_makefile)
    sys.stdout.flush()

    c2cupgradetool = C2cUpgradeTool(options)
    c2cupgradetool.upgrade()


def _fill_arguments():
    parser = ArgumentParser()
    parser.add_argument(
        "--windows",
        action="store_true",
        help="Use the windows c2cgeoportal package",
    )
    parser.add_argument(
        "--nondocker",
        action="store_true",
        help="Use the nondocker upgrade",
    )
    parser.add_argument(
        "--force-docker",
        action="store_true",
        help="Disable the nondocker upgrade",
    )
    parser.add_argument(
        "--git-remote",
        metavar="GITREMOTE",
        help="Specify the remote branch",
        default="origin",
    )
    parser.add_argument(
        "--makefile", help="The makefile used to build", default="Makefile",
    )
    parser.add_argument(
        "--new-makefile", help="The makefile used to override the makefile",
    )
    parser.add_argument(
        "--use-makefile", action="store_true", help="c2cupgrade is running from a makefile",
    )
    parser.add_argument(
        "--step", type=int, help=argparse.SUPPRESS, default=0
    )

    return parser


class InteruptedException(Exception):
    pass


current_step_number = 0


class Step:
    def __init__(self, step_number, file_marker=True):
        global current_step_number
        current_step_number = step_number
        self.step_number = step_number
        self.file_marker = file_marker

    def __call__(self, current_step):
        def decorate(c2cupgradetool, *args, **kwargs):
            try:
                if os.path.isfile(".UPGRADE{}".format(self.step_number - 1)):
                    os.unlink(".UPGRADE{}".format(self.step_number - 1))
                if self.file_marker:
                    with open(".UPGRADE{}".format(self.step_number), "w"):
                        pass
                print("Start step {}.".format(self.step_number))
                sys.stdout.flush()
                current_step(c2cupgradetool, self.step_number, *args, **kwargs)
            except subprocess.CalledProcessError as e:
                c2cupgradetool.print_step(
                    self.step_number, error=True,
                    message="The command `./docker-run {}` returns the error code {}.".format(
                        " ".join(["'{}'".format(e) for e in e.cmd]), e.returncode
                    ),
                    prompt="Fix the error and run the step again:"
                )
                exit(1)
            except InteruptedException as e:
                c2cupgradetool.print_step(
                    self.step_number, error=True,
                    message="There was an error: {}.".format(e),
                    prompt="Fix the error and run the step again:"
                )
                exit(1)
            except Exception as e:
                ex = e

                global current_step_number
                if self.step_number == current_step_number:
                    def message():
                        c2cupgradetool.print_step(
                            self.step_number, error=True,
                            message="The step had the error '{}'.".format(ex),
                            prompt="Fix the error and run the step again:"
                        )
                    atexit.register(message)
                raise
        return decorate


class C2cUpgradeTool:

    color_bar = colorize("================================================================", GREEN)
    project = None

    def __init__(self, options):
        self.options = options
        self.project = self.get_project()

    @staticmethod
    def get_project():
        if not os.path.isfile("project.yaml"):
            print(colorize("Unable to find the required 'project.yaml' file.", RED))
            exit(1)

        with open("project.yaml", "r") as f:
            return yaml.safe_load(f)

    @staticmethod
    def get_upgrade(section):
        if not os.path.isfile(".upgrade.yaml"):
            print(colorize("Unable to find the required '.upgrade.yaml' file.", RED))
            exit(1)

        with open(".upgrade.yaml", "r") as f:
            return yaml.safe_load(f)[section]

    def print_step(self, step, error=False, message=None, prompt="To continue, type:"):
        print("")
        print(self.color_bar)
        if message is not None:
            print(colorize(message, RED if error else YELLOW))
        if step >= 0:
            print(colorize(prompt, GREEN))
            if self.options.use_makefile:
                args = " --makefile={}".format(self.options.makefile) \
                    if self.options.makefile != "Makefile" else ""
                print(colorize("./docker-run --home make{} upgrade{}".format(
                    args, step if step != 0 else "",
                ), GREEN))
            else:
                cmd = [
                    "./docker-run",
                    "--home",
                    "--image=camptocamp/geomapfish-build",
                    "--version=" + pkg_resources.get_distribution("c2cgeoportal_commons").version,
                    "c2cupgrade",
                ]
                if self.options.windows:
                    cmd.append("--windows")
                if self.options.nondocker:
                    cmd.append("--nondocker")
                if self.options.force_docker:
                    cmd.append("--force-docker")
                if self.options.git_remote != "origin":
                    cmd.append("--git-remote={}".format(self.options.git_remote))
                if self.options.makefile != "Makefile":
                    cmd.append("--makefile={}".format(self.options.makefile))
                if self.options.new_makefile != self.options.makefile:
                    cmd.append("--new-makefile={}".format(self.options.new_makefile))
                if step != 0:
                    cmd.append("--step={}".format(step))
                print(colorize(" ".join(cmd), GREEN))

    def run_step(self, step):
        getattr(self, "step{}".format(step))()

    def test_checkers(self):
        try:
            resp = requests.get(
                self.project["checker_url"],
                headers=self.project.get("checker_headers"),
                verify=False
            )
        except ConnectionRefusedError as e:
            return False, "\n".join([
                "Connection refused: {}",
                "Run `curl --insecure {} '{}'` for more information."
            ]).format(
                e,
                ' '.join([
                    '--header {}={}'.format(*i) for i in self.project.get("checker_headers", {}).items()
                ]),
                self.project["checker_url"],
            )
        if resp.status_code < 200 or resp.status_code >= 300:
            return False, "\n".join([
                "Checker error:",
                "Run `curl {} '{}'` for more information."
            ]).format(
                ' '.join([
                    '--header {}={}'.format(*i) for i in self.project.get("checker_headers", {}).items()
                ]),
                self.project["checker_url"],
            )

        return True, None

    def upgrade(self):
        self.run_step(self.options.step)

    @Step(0, file_marker=False)
    def step0(self, step):
        project_template_keys = list(self.project.get("template_vars").keys())
        messages = []
        for required in REQUIRED_TEMPLATE_KEYS:
            if required not in project_template_keys:
                messages.append(
                    "The element '{}' is missing in the 'template_vars' of "
                    "the file 'project.yaml.mako', you should have for example: {}: {}.".format(
                        required, required, TEMPLATE_EXAMPLE.get('required', '')
                    )
                )
        if self.project.get("managed_files") is None:
            messages.append(
                "The element 'managed_files' is missing in the file 'project.yaml.mako', "
                "you must define this element with a list of regular expressions or with an empty array. "
                "See upgrade documentation for more information."
            )
        if len(messages) > 0:
            self.print_step(
                step, error=True, message="\n".join(messages),
                prompt="Fix it and run again the upgrade:")
            exit(1)

        if check_git_status_output() == "":
            self.run_step(step + 1)
        else:
            check_call(["git", "status"])
            self.print_step(
                step + 1, message="Here is the output of 'git status'. Please make sure to commit all your "
                "changes before going further. All uncommitted changes will be lost."
            )

    @Step(1)
    def step1(self, step):
        shutil.copyfile("project.yaml", "/tmp/project.yaml")
        try:
            check_call(["git", "reset", "--hard"])
            check_call(["git", "clean", "--force", "-d"])
        finally:
            shutil.copyfile("/tmp/project.yaml", "project.yaml")

        self.run_step(step + 1)

    @Step(2)
    def step2(self, step):
        project_path = os.path.join("/tmp", self.project["project_folder"])
        check_call([
            "pcreate", "--ignore-conflicting-name", "--overwrite",
            "--scaffold=c2cgeoportal_update", project_path
        ])
        if self.options.nondocker:
            check_call([
                "pcreate", "--ignore-conflicting-name", "--overwrite",
                "--scaffold=c2cgeoportal_nondockerupdate", project_path
            ])

        shutil.copyfile(os.path.join(project_path, ".upgrade.yaml"), ".upgrade.yaml")
        self.files_to_move(prefix="CONST_create_template", force=True)

        shutil.rmtree(project_path)
        os.remove(".upgrade.yaml")

        # To be removed in 2.4
        with open("CONST_create_template/vars.yaml", "r") as f:
            new_syntax = f.readline() == "---\n"
        if not new_syntax:
            check_call(["sed", "--in-place", "s/- /  - /g", "CONST_create_template/vars.yaml"])
            check_call(["sed", "--in-place", "s/    /  /g", "CONST_create_template/vars.yaml"])

        check_call(["git", "add", "--all", "--force", "CONST_create_template/"])
        call(["git", "commit", "--message=Perform the move into the CONST_create_template folder"])

        self.run_step(step + 1)

    @Step(3)
    def step3(self, step):
        if os.path.exists("CONST_create_template"):
            check_call(["git", "rm", "-r", "--force", "CONST_create_template/"])

        project_path = os.path.join("/tmp", self.project["project_folder"])
        check_call(["ln", "-s", "/src", project_path])
        check_call([
            "pcreate", "--ignore-conflicting-name", "--overwrite",
            "--scaffold=c2cgeoportal_update", project_path
        ])
        if self.options.nondocker:
            check_call([
                "pcreate", "--ignore-conflicting-name", "--overwrite",
                "--scaffold=c2cgeoportal_nondockerupdate", project_path
            ])
        os.remove(project_path)

        check_call(["git", "add", "--all", "CONST_create_template/"])
        check_call(["git", "clean", "-Xf", "CONST_create_template/"])
        shutil.copyfile("project.yaml", "/tmp/project.yaml")
        try:
            check_call(["make", "--makefile=" + self.options.makefile, "clean-all"])
        finally:
            shutil.copyfile("/tmp/project.yaml", "project.yaml")
        self.run_step(step + 1)

    @Step(4)
    def step4(self, step):
        error = False

        print("Files to remove")
        error |= self.files_to_remove(pre=True)
        print("Files to move")
        error |= self.files_to_move(pre=True)
        print("Files to get")
        error |= self.files_to_get(step, pre=True)

        if error:
            self.print_step(
                step, error=True, message="There is an error in your project configuration, see above",
                prompt="Fix it and run the step again:"
            )
            exit(1)
        elif "managed_files" not in self.project:
            self.print_step(
                step,
                message="In the new version, we will also manage almost all the create "
                "template files.\n"
                "By default, files conforming to the following regex pattern will not be replaced:\n{}"
                "Therefore, you should fill the 'managed_files' in you 'project.yaml' file with at least "
                "`[]`.".format("\n".join([
                    "- " + e for e in self.project.get('unmanaged_files', [])
                ])),
                prompt="Fill it and run the step again:"
            )
        else:
            self.run_step(step + 1)

    @Step(5)
    def step5(self, step):
        self.files_to_remove()
        self.run_step(step + 1)

    def files_to_remove(self, pre=False):
        error = False
        for element in self.get_upgrade("files_to_remove"):
            file_ = element["file"]
            if os.path.exists(file_):
                managed = False
                for pattern in self.project["managed_files"]:
                    if re.match(pattern + '$', file_):
                        print(colorize(
                            "The file '{}' is no longer used, but not deleted "
                            "because it is in the managed_files as '{}'.".format(file_, pattern),
                            RED
                        ))
                        managed = True
                if not managed and not pre:
                    print(colorize("The file '{}' is removed.".format(file_), GREEN))
                    if "version" in element and "from" in element:
                        print("Was used in version {}, to be removed from version {}.".format(
                            element["from"], element["version"]
                        ))
                    if os.path.isdir(file_):
                        shutil.rmtree(file_)
                    else:
                        os.remove(file_)
        return error

    @Step(6)
    def step6(self, step):
        self.files_to_move()
        self.run_step(step + 1)

    def files_to_move(self, pre=False, prefix="", force=False):
        error = False
        for element in self.get_upgrade("files_to_move"):
            src = os.path.join(prefix, element["from"])
            dst = os.path.join(prefix, element["to"])
            if os.path.exists(src):
                managed = False
                type_ = "directory" if os.path.isdir(src) else "file"
                if not force:
                    for pattern in self.project["managed_files"]:
                        if re.match(pattern + '$', src):
                            print(colorize(
                                "The {} '{}' is present in the managed_files as '{}', but it will move."
                                .format(
                                    type_, src, pattern
                                ),
                                RED
                            ))
                            error = True
                            managed = True
                            break
                        if re.match(pattern + '$', dst):
                            print(colorize(
                                "The {} '{}' is present in the managed_files as '{}', but it will move."
                                .format(
                                    type_, dst, pattern
                                ),
                                RED
                            ))
                            error = True
                            managed = True
                            break
                if not managed and os.path.exists(dst):
                    print(colorize(
                        "The destination '{}' already exists, ignoring.".format(dst),
                        YELLOW
                    ))
                if not managed and not pre:
                    print(colorize("Move the {} '{}' to '{}'.".format(type_, src, dst), GREEN))
                    if "version" in element:
                        print("Needed from version {}.".format(element["version"]))
                    if os.path.dirname(dst) != "":
                        os.makedirs(os.path.dirname(dst), exist_ok=True)
                    os.rename(src, dst)
        return error

    @Step(7)
    def step7(self, step):
        self.files_to_get(step)
        self.run_step(step + 1)

    def is_managed(self, file_, files_to_get=False):
        default_project_file = self.get_upgrade('default_project_file')

        # Managed means managed by the application owner, not the c2cupgrade
        managed = False
        if not files_to_get or os.path.exists(file_) or not \
                check_git_status_output(["CONST_create_template/" + file_]).startswith("A  "):
            for pattern in default_project_file['include']:
                if re.match(pattern + '$', file_):
                    print("File '{}' included by migration config pattern '{}'.".format(file_, pattern))
                    managed = True
                    break
            if managed:
                for pattern in default_project_file['exclude']:
                    if re.match(pattern + '$', file_):
                        print("File '{}' excluded by migration config pattern '{}'.".format(file_, pattern))
                        print('managed', file_, pattern)
                        managed = False
                        break
        else:
            print("New file '{}'.".format(file_))

        if not managed and not os.path.exists(file_):
            for pattern in self.get_upgrade('extra'):
                if re.match(pattern + '$', file_):
                    print("File '{}' is an extra by migration config pattern '{}'.".format(file_, pattern))
                    managed = True

        if not managed:
            for pattern in self.project['managed_files']:
                if re.match(pattern + '$', file_):
                    print("File '{}' included by project config pattern (managed_files) '{}'.".format(
                        file_, pattern
                    ))
                    print('managed', file_, pattern)
                    managed = True
                    break
        if managed:
            for pattern in self.project.get('unmanaged_files', []):
                if re.match(pattern + '$', file_):
                    print("File '{}' excluded by project config pattern (unmanaged_files) '{}'.".format(
                        file_, pattern
                    ))
                    managed = False
                    break

        return managed

    def files_to_get(self, step, pre=False):
        error = False
        for root, _, files in os.walk("CONST_create_template"):
            root = root[len("CONST_create_template/"):]
            for file_ in files:
                destination = os.path.join(root, file_)
                managed = self.is_managed(destination, True)
                source = os.path.join("CONST_create_template", destination)
                if not managed and (not os.path.exists(destination) or not filecmp.cmp(source, destination)):
                    print(colorize(
                        "Get the file '{}' from the create template.".format(destination), GREEN
                    ))
                    if not pre:
                        if os.path.dirname(destination) != "":
                            os.makedirs(os.path.dirname(destination), exist_ok=True)
                        try:
                            shutil.copyfile(source, destination)
                            shutil.copymode(source, destination)
                        except PermissionError as e:
                            self.print_step(
                                step, error=True, message=(
                                    "All your project files should be owned by your user, "
                                    "current error:\n" + str(e)
                                ),
                                prompt="Fix it and run the upgrade again:")
                            exit(1)
                elif managed:
                    print("The file '{}' is managed by the project.".format(destination))
                elif os.path.exists(destination) and filecmp.cmp(source, destination):
                    print("The file '{}' does not change.".format(destination))
                else:
                    print("Unknown stat for the file '{}'.".format(destination))
                    exit(2)
        return error

    @Step(8)
    def step8(self, step):
        with open("changelog.diff", "w") as diff_file:
            check_call(["git", "diff", "--", "CONST_CHANGELOG.txt"], stdout=diff_file)

        from210 = False
        try:
            check_call(["grep", "--", "-Version 2.1.0", "changelog.diff"])
            from210 = True
        except subprocess.CalledProcessError:
            pass
        if from210:
            check_call(["cp", "CONST_CHANGELOG.txt", "changelog.diff"])

        if os.path.getsize("changelog.diff") == 0:
            self.run_step(step + 1)
        else:
            self.print_step(
                step + 1,
                message="Apply the manual migration steps based on what is in the CONST_CHANGELOG.txt "
                "file (listed in the `changelog.diff` file)."
            )

    def get_modified(self, status_path):
        status = check_git_status_output([status_path])
        status = [s for s in status.split("\n") if len(s) > 3]
        status = [s[3:] for s in status if s[:3].strip() == "M"]
        for pattern in self.get_upgrade('no_diff'):
            matcher = re.compile('CONST_create_template/{}$'.format(pattern))
            status = [s for s in status if not matcher.match(s)]
        status = [s for s in status if os.path.exists(s[len("CONST_create_template/"):])]
        status = [s for s in status if not filecmp.cmp(s, s[len("CONST_create_template/"):])]
        return status

    @Step(9)
    def step9(self, step):
        if os.path.isfile("changelog.diff"):
            os.unlink("changelog.diff")

        status = self.get_modified("CONST_create_template/geoportal/{}_geoportal/static-ngeo".format(
            self.project["project_package"]
        ))

        with open("ngeo.diff", "w") as diff_file:
            if len(status) != 0:
                check_call(
                    ["git", "diff", "--relative=CONST_create_template", "--staged", "--"] + status,
                    stdout=diff_file)

        if os.path.getsize("ngeo.diff") == 0:
            self.run_step(step + 1)
        else:
            self.print_step(
                step + 1,
                message="Manually apply the ngeo application changes as shown in the `ngeo.diff` file.\n"
                + DIFF_NOTICE
                + "\nNote that you can also apply them using: git apply --3way ngeo.diff"
            )

    @Step(10)
    def step10(self, step):
        if os.path.isfile("ngeo.diff"):
            os.unlink("ngeo.diff")

        status = self.get_modified("CONST_create_template")
        status = [s for s in status if not s.startswith(
            "CONST_create_template/geoportal/{}_geoportal/static-ngeo/".format(
                self.project["project_package"]
            ),
        )]

        self.options.use_makefile = True
        self.options.makefile = self.options.new_makefile

        if len(status) > 0:
            with open("create.diff", "w") as diff_file:
                if len(status) != 0:
                    check_call(
                        ["git", "diff", "--relative=CONST_create_template", "--staged", "--"] + status,
                        stdout=diff_file)

            if os.path.getsize("create.diff") == 0:
                self.run_step(step + 1)
            else:
                self.print_step(
                    step + 1, message="The `create.diff` file is a recommendation of the changes that you "
                    "should apply to your project.\n" + DIFF_NOTICE
                    + "\nNote that you can also apply them using: git apply --3way create.diff"
                )
        else:
            self.run_step(step + 1)

    @Step(11)
    def step11(self, step):
        if os.path.isfile("create.diff"):
            os.unlink("create.diff")

        check_call(["make", "--makefile=" + self.options.new_makefile, "build"])

        if self.options.nondocker:
            command.upgrade(Config("geoportal/alembic.ini", ini_section="main"), "head")
            command.upgrade(Config("geoportal/alembic.ini", ini_section="static"), "head")

            args = " --makefile={}".format(self.options.makefile) \
                if self.options.makefile != "Makefile" else ""
            message = [
                "The upgrade is nearly done, now you should:",
                "- Run the finalisation build with `FINALISE=TRUE make{} build`.".format(args),
                "- Test your application."
            ]
        else:
            message = [
                "The upgrade is nearly done, now you should:",
                "- To upgrade the database run `./docker-compose-run alembic --name=main "
                "--config=geoportal/alembic.ini upgrade head`",
                "- Run `DOCKER_TAG=unexisting docker-compose pull --ignore-pull-failures && "
                "docker-compose down --remove-orphans && docker-compose up -d`.",
                "- Test your application on '{}'.".format(
                    self.project.get('application_url', '... missing ...')
                )
            ]

        if self.options.windows:
            message.append(
                "You are running on Windows, please restart your Apache server,"
                "because we can not do that automatically."
            )

        if os.path.isfile(".upgrade.yaml"):
            os.unlink(".upgrade.yaml")
        with open(".UPGRADE_SUCCESS", "w"):
            pass
        self.print_step(step + 1, message="\n".join(message))

    @Step(12, file_marker=False)
    def step12(self, step):
        if os.path.isfile(".UPGRADE_SUCCESS"):
            os.unlink(".UPGRADE_SUCCESS")
        ok, message = self.test_checkers()
        if ok:
            self.run_step(step + 1)
        else:
            self.print_step(
                step, error=True, message=message,
                prompt="Correct the checker, then run the step again "
                       "(If you want to fix it later you can pass to the next step):"
            )
            exit(1)

    @Step(13, file_marker=False)
    def step13(self, step):
        # Required to remove from the Git stage the ignored file when we lunch the step again
        check_call(["git", "reset", "--mixed"])

        check_call(["git", "add", "-A"])
        check_call(["git", "status"])

        self.print_step(
            step + 1, message="We will commit all the above files!\n"
            "If there are some files which should not be committed, then you should "
            "add them into the `.gitignore` file and launch upgrade{} again.".format(step),
            prompt="Then to commit your changes type:")

    @Step(14, file_marker=False)
    def step14(self, _):
        check_call(["git", "commit", "--message=Upgrade to GeoMapFish {}".format(
            pkg_resources.get_distribution("c2cgeoportal_commons").version
        )])

        print("")
        print(self.color_bar)
        print("")
        print(colorize("Congratulations, your upgrade was successful.", GREEN))
        print("")
        branch = check_output(["git", "rev-parse", "--abbrev-ref", "HEAD"]).decode("utf-8").strip()
        print("Now all your files are committed; you should do a git push:")
        print("git push {0!s} {1!s}.".format(
            self.options.git_remote, branch
        ))


def check_git_status_output(args=[]):
    return check_output(["git", "status", "--short"] + args).decode("utf-8")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2017, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

import logging
import subprocess
import c2cwsgiutils.health_check
from time import sleep
from urllib.parse import urljoin

import requests

log = logging.getLogger(__name__)


def build_url(name, path, request, headers=None):
    base_internal_url = request.registry.settings["checker"]["base_internal_url"]
    url = urljoin(base_internal_url, path)

    forward_host = request.registry.settings["checker"].get("forward_host", False)
    headers = _build_headers(request, headers)
    if forward_host:
        headers["Host"] = request.host

    log.debug("%s, URL: %s", name, url)
    return {"url": url, "headers": headers}


def _build_headers(request, headers=None):
    if headers is None:
        headers = {}
    headers["Cache-Control"] = "no-cache"
    settings = request.registry.settings.get("checker", {})
    for header in settings.get("forward_headers", []):
        value = request.headers.get(header)
        if value is not None:
            headers[header] = value
    return headers


def _routes(settings, health_check):
    routes_settings = settings["routes"]
    for route in routes_settings["routes"]:
        if route.get("checker_name", route["name"]) not in routes_settings["disable"]:
            name = "checker_routes_" + route.get("checker_name", route["name"])

            class GetRequest:
                """
                Get the request information about the current route name
                """
                def __init__(self, route_name, type_):
                    self.route_name = route_name
                    self.type = type_

                def __call__(self, request):
                    return build_url("route", request.route_path(self.route_name), request)[self.type]

            health_check.add_url_check(
                url=GetRequest(route['name'], 'url'),
                name=name,
                params=route.get("params", None),
                headers=GetRequest(route['name'], 'headers'),
                level=route["level"],
                timeout=30,
            )


def _pdf3(settings, health_check):
    print_settings = settings["print"]

    def check(request):
        path = request.route_path("printproxy_report_create", format="pdf")
        url_headers = build_url("Check the printproxy request (create)", path, request)

        session = requests.session()
        resp = session.post(
            json=print_settings["spec"],
            timeout=30,
            **url_headers
        )
        resp.raise_for_status()

        job = resp.json()

        path = request.route_path("printproxy_status", ref=job["ref"])
        url_headers = build_url("Check the printproxy pdf status", path, request)
        done = False
        while not done:
            sleep(1)
            resp = session.get(
                timeout=30,
                **url_headers
            )
            resp.raise_for_status()

            status = resp.json()
            if "error" in status:
                raise Exception("Failed to do the printing: {0!s}".format(status["error"]))
            done = status["done"]

        path = request.route_path("printproxy_report_get", ref=job["ref"])
        url_headers = build_url("Check the printproxy pdf retrieve", path, request)
        resp = session.get(
            timeout=30,
            **url_headers
        )
        resp.raise_for_status()

    health_check.add_custom_check(name="checker_print", check_cb=check, level=print_settings["level"])


def _fts(settings, health_check):
    fts_settings = settings["fulltextsearch"]
    if fts_settings.get("disable", False):
        return

    def get_both(request):
        return build_url("Check the fulltextsearch", request.route_path("fulltextsearch"), request)

    def check(_request, response):
        assert len(response.json()["features"]) > 0, "No result"

    health_check.add_url_check(
        name="checker_fulltextsearch",
        url=lambda r: get_both(r)["url"],
        headers=lambda r: get_both(r)["headers"],
        params={
            "query": fts_settings["search"],
            "limit": "1",
        },
        check_cb=check,
    )


def _themes_errors(settings, health_check):
    from c2cgeoportal_commons.models import DBSession
    from c2cgeoportal_commons.models.main import Interface

    themes_settings = settings["themes"]
    default_params = themes_settings.get("params", {})
    interfaces_settings = themes_settings["interfaces"]

    def check(request):
        path = request.route_path("themes")
        session = requests.session()
        for interface, in DBSession.query(Interface.name).all():
            params = {}
            params.update(default_params)
            params.update(interfaces_settings.get(interface, {}).get("params", {}))
            params["interface"] = interface

            interface_url_headers = build_url("checker_themes " + interface, path, request)

            response = session.get(
                params=params,
                timeout=120,
                **interface_url_headers
            )
            response.raise_for_status()

            result = response.json()
            if len(result["errors"]) != 0:
                raise c2cwsgiutils.health_check.JsonCheckException(
                    "Interface '{}' has error in Theme.".format(interface),
                    result["errors"]
                )

    health_check.add_custom_check(name="checker_themes", check_cb=check, level=themes_settings["level"])


def _lang_files(global_settings, settings, health_check):
    lang_settings = settings["lang"]
    available_locale_names = global_settings["available_locale_names"]

    default_name = global_settings["default_locale_name"]
    assert \
        default_name in available_locale_names, \
        "default_locale_name '{}' not in available_locale_names: {}".format(
            default_name, ", ".join(available_locale_names)
        )

    for type_ in lang_settings.get("files", []):
        for lang in available_locale_names:
            if type_ == "cgxp-api":
                url = "{package}_geoportal:static/build/api-lang-{lang}.js"
            elif type_ == "ngeo":
                url = "{package}_geoportal:static-ngeo/build/{lang}.json"
            else:
                raise Exception("Your language type value '%s' is not valid, "
                                "available values [cgxp-api, ngeo]" % type_)
            name = "checker_lang_{}_{}".format(type_, lang)

            def get_both(url, lang, request):
                return build_url(
                    name,
                    request.static_path(url.format(package=global_settings["package"], lang=lang)),
                    request
                )

            health_check.add_url_check(
                name=name,
                url=lambda r, u=url, la=lang: get_both(u, la, r)["url"],
                headers=lambda r, u=url, la=lang: get_both(u, la, r)["headers"],
                level=lang_settings["level"],
            )


def _phantomjs(settings, health_check):
    phantomjs_settings = settings["phantomjs"]
    for route in phantomjs_settings["routes"]:
        if route.get("checker_name", route["name"]) in phantomjs_settings["disable"]:
            continue

        def check(request):
            path = request.route_path(route["name"], _query=route.get("params", {}))
            url = build_url("Check", path, request)["url"]

            cmd = [
                "phantomjs", "--local-to-remote-url-access=true",
                "/usr/lib/node_modules/ngeo/buildtools/check-example.js", url
            ]

            try:
                subprocess.check_output(cmd, timeout=10)
            except subprocess.CalledProcessError as e:
                raise Exception(e.output.decode("utf-8"))
            except subprocess.TimeoutExpired as e:
                raise Exception("""Timeout:
command: {}
output:
{}""".format(" ".join(e.cmd), e.output.decode("utf-8")))
        name = "checker_phantomjs_" + route.get("checker_name", route["name"])
        health_check.add_custom_check(name=name, check_cb=check, level=route["level"])


def init(config, health_check):
    global_settings = config.get_settings()
    if "checker" not in global_settings:
        return
    settings = global_settings["checker"]
    _routes(settings, health_check)
    _pdf3(settings, health_check)
    _fts(settings, health_check)
    _themes_errors(settings, health_check)
    _lang_files(global_settings, settings, health_check)
    _phantomjs(settings, health_check)



# -*- coding: utf-8 -*-

# Copyright (c) 2012-2017, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.
import logging

import requests

from c2cgeoportal_geoportal.lib.checker import build_url

LOG = logging.getLogger(__name__)


def init(config, health_check):
    global_settings = config.get_settings()
    if "check_collector" not in global_settings:
        return
    settings = global_settings["check_collector"]
    c2c_base = global_settings.get("c2c.base_path", "")

    max_level = settings["max_level"]

    for host in settings["hosts"]:
        def check(request):
            params = request.params
            display = host["display"]
            if "host" not in params or display == params["host"]:
                url_headers = build_url(
                    "check_collector",
                    "%s/%s/health_check" % (host["url"].rstrip("/"), c2c_base.strip("/")),
                    request
                )
                r = requests.get(
                    params={"max_level": str(host.get("max_level", max_level))},
                    timeout=120,
                    **url_headers
                )
                r.raise_for_status()
                return r.json()

        health_check.add_custom_check(
            name="check_collector_" + host["display"],
            check_cb=check,
            level=settings["level"]
        )



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import sys
import urllib.parse
import logging
import requests

from pyramid.response import Response
from pyramid.httpexceptions import HTTPBadGateway, exception_response

from c2cgeoportal_geoportal.lib.caching import get_region, set_common_headers, \
    NO_CACHE, PUBLIC_CACHE, PRIVATE_CACHE


log = logging.getLogger(__name__)
cache_region = get_region()


class Proxy(object):

    def __init__(self, request):
        self.request = request
        self.host_forward_host = request.registry.settings["host_forward_host"]
        self.http_options = self.request.registry.settings.get("http_options", {})

    def _proxy(self, url, params=None, method=None, cache=False, body=None, headers=None):
        # get query string
        params = dict(self.request.params) if params is None else params
        parsed_url = urllib.parse.urlparse(url)
        all_params = urllib.parse.parse_qs(parsed_url.query)
        for p in all_params:  # pragma: no cover
            all_params[p] = ",".join(all_params[p])
        all_params.update(params)
        query_string = urllib.parse.urlencode(all_params)

        if parsed_url.port is None:
            url = "{0!s}://{1!s}{2!s}?{3!s}".format(
                parsed_url.scheme, parsed_url.hostname,
                parsed_url.path, query_string
            )
        else:  # pragma: no cover
            url = "{0!s}://{1!s}:{2:d}{3!s}?{4!s}".format(
                parsed_url.scheme, parsed_url.hostname, parsed_url.port,
                parsed_url.path, query_string
            )

        log.info("Send query to URL:\n{0!s}.".format(url))

        if method is None:
            method = self.request.method

        if headers is None:  # pragma: no cover
            headers = dict(self.request.headers)

        # Forward request to target (without Host Header)
        if parsed_url.hostname not in self.host_forward_host and "Host" in headers:  # pragma: no cover
            headers.pop("Host")

        # Forward the request tracking ID to the other service. This will allow to follow the logs belonging
        # to a single request coming from the user
        headers.setdefault('X-Request-ID', self.request.c2c_request_id)

        if not cache:
            headers["Cache-Control"] = "no-cache"

        if method in ("POST", "PUT") and body is None:  # pragma: no cover
            body = self.request.body

        try:
            if method in ("POST", "PUT"):
                response = requests.request(
                    method, url, data=body, headers=headers, **self.http_options
                )
            else:
                response = requests.request(
                    method, url, headers=headers, **self.http_options
                )
        except Exception:  # pragma: no cover
            errors = [
                "Error '%s' while getting the URL:",
                "%s",
                "Method: %s",
                "--- With headers ---",
                "%s",
            ]
            args = [
                sys.exc_info()[0], url, method,
                "\n".join(["{}: {}".format(*h) for h in list(headers.items())]),
            ]
            if method in ("POST", "PUT"):
                errors += [
                    "--- Query with body ---",
                    "%s",
                ]
                args.append(body.decode("utf-8"))
            log.error("\n".join(errors), *args, exc_info=True)

            raise HTTPBadGateway("Error on backend, See logs for detail")

        if not response.ok:  # pragma: no cover
            errors = [
                "Error '%s' in response of URL:",
                "%s",
                "Status: %d",
                "Method: %s",
                "--- With headers ---",
                "%s",
            ]
            args = [
                response.reason, url, response.status_code, method,
                "\n".join(["{}: {}".format(*h) for h in list(headers.items())]),
            ]
            if method in ("POST", "PUT"):
                errors += [
                    "--- Query with body ---",
                    "%s",
                ]
                args.append(body.decode("utf-8"))
            errors += [
                "--- Return content ---",
                "%s",
            ]
            args.append(response.text)
            log.error("\n".join(errors), *args)

            raise exception_response(response.status_code)

        return response

    @cache_region.cache_on_arguments()
    def _proxy_cache(self, method, *args, **kwargs):  # pragma: no cover
        # method is only for the cache
        del method
        return self._proxy(*args, cache=True, **kwargs)

    def _proxy_response(
        self, service_name, url,
        headers_update=None, public=False, **kwargs
    ):  # pragma: no cover
        if headers_update is None:
            headers_update = {}
        cache = kwargs.get("cache", False)
        if cache is True:
            response = self._proxy_cache(
                url,
                self.request.method,
                **kwargs
            )
        else:
            response = self._proxy(url, **kwargs)

        cache_control = (PUBLIC_CACHE if public else PRIVATE_CACHE) if cache else NO_CACHE
        return self._build_response(
            response, response.content, cache_control, service_name,
            headers_update=headers_update
        )

    def _build_response(
        self, response, content, cache_control, service_name,
        headers=None, headers_update=None, content_type=None
    ):
        if isinstance(content, str):
            content = content.encode("utf-8")
        if headers_update is None:
            headers_update = {}
        headers = response.headers if headers is None else headers

        # Hop-by-hop Headers are not supported by WSGI
        # See:
        # https://www.python.org/dev/peps/pep-3333/#other-http-features
        # chapter 13.5.1 at http://www.faqs.org/rfcs/rfc2616.html
        for header in [
                "Connection",
                "Keep-Alive",
                "Proxy-Authenticate",
                "Proxy-Authorization",
                "te",
                "Trailers",
                "Transfer-Encoding",
                "Upgrade",
        ]:  # pragma: no cover
            if header in headers:
                del headers[header]
        # Other problematic headers
        for header in [
            "Content-Length",
            "Content-Location",
            "Content-Encoding",
        ]:  # pragma: no cover
            if header in headers:
                del headers[header]

        headers.update(headers_update)

        response = Response(content, status=response.status_code, headers=headers)

        return set_common_headers(
            self.request, service_name, cache_control,
            response=response,
            content_type=content_type,
        )

    @staticmethod
    def _get_lower_params(params):
        return dict(
            (k.lower(), str(v).lower()) for k, v in params.items()
        )

    def _get_headers(self):
        headers = self.request.headers
        if "Cookie" in headers:  # pragma: no cover
            headers.pop("Cookie")
        return headers



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (c) 2013-2017, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

import os

from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))
README = """c2cgeoportal is the server part of `GeoMapFish <http://geomapfish.org/>`_,
the client part is `ngeo <https://github.com/camptocamp/ngeo/>`_,
the old client and API part is `CGXP <https://github.com/camptocamp/cgxp/>`_.

Read the `Documentation <https://camptocamp.github.io/c2cgeoportal/master/>`_.

`Sources <https://github.com/camptocamp/c2cgeoportal/>`_"""
VERSION = os.environ.get('VERSION', 'dev')


with open(os.path.join(HERE, 'requirements.txt')) as f:
    install_requires = f.read().splitlines()

setup_requires = []

tests_require = []

setup(
    name='c2cgeoportal_geoportal',
    version=VERSION,
    description='c2cgeoportal geoportal',
    long_description=README,
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Pyramid',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],
    author='Camptocamp',
    author_email='info@camptocamp.com',
    url='http://www.camptocamp.com/solutions/geospatial/',
    keywords='web gis geoportail c2cgeoportal geocommune pyramid',
    packages=find_packages(exclude=['tests.*']),
    package_data={'c2cgeoportal_geoportal': ['py.typed']},
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    entry_points={
        'console_scripts': [
            'manage-users = c2cgeoportal_geoportal.scripts.manage_users:main',
            'c2cupgrade = c2cgeoportal_geoportal.scripts.c2cupgrade:main',
            'db2pot = c2cgeoportal_geoportal.scripts.db2pot:main',
            'themev1tov2 = c2cgeoportal_geoportal.scripts.themev1tov2:main',
            'theme2fts = c2cgeoportal_geoportal.scripts.theme2fts:main',
            'l10nv1tov2 = c2cgeoportal_geoportal.scripts.l10nv1tov2:main',
            'create-demo-theme = c2cgeoportal_geoportal.scripts.create_demo_theme:main',
            'treeitem-uniquename = c2cgeoportal_geoportal.scripts.treeitem_uniquename:main',
            'urllogin = c2cgeoportal_geoportal.scripts.urllogin:main',
        ],
        'pyramid.scaffold': [
            'c2cgeoportal_create = c2cgeoportal_geoportal.scaffolds:TemplateCreate',
            'c2cgeoportal_update = c2cgeoportal_geoportal.scaffolds:TemplateUpdate',
            'c2cgeoportal_nondockercreate = c2cgeoportal_geoportal.scaffolds:TemplateNondockerCreate',
            'c2cgeoportal_nondockerupdate = c2cgeoportal_geoportal.scaffolds:TemplateNondockerUpdate',
        ],
        'lingua.extractors': [
            'geomapfish-theme = c2cgeoportal_geoportal.lib.lingua_extractor:GeomapfishThemeExtractor',
            'geomapfish-config = c2cgeoportal_geoportal.lib.lingua_extractor:GeomapfishConfigExtractor',
            'geomapfish-angular = c2cgeoportal_geoportal.lib.lingua_extractor:GeomapfishAngularExtractor',
        ],
        'plaster.loader_factory': [
            'c2cgeoportal=c2cgeoportal_geoportal.lib.loader:Loader',
            'c2cgeoportal+ini=c2cgeoportal_geoportal.lib.loader:Loader',
            'c2cgeoportal+egg=c2cgeoportal_geoportal.lib.loader:Loader',
        ],
        'plaster.wsgi_loader_factory': [
            'c2cgeoportal=c2cgeoportal_geoportal.lib.loader:Loader',
            'c2cgeoportal+ini=c2cgeoportal_geoportal.lib.loader:Loader',
            'c2cgeoportal+egg=c2cgeoportal_geoportal.lib.loader:Loader',
        ],
    }
)



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import os
from urllib.parse import urlsplit, urlunsplit
from pyramid.scripts.common import get_config_loader


def fill_arguments(parser):
    default_app_config = "geoportal/production.ini" \
        if os.path.isfile("geoportal/production.ini") \
        else "production.ini"

    parser.add_argument(
        "--app-config", "-i",
        default=default_app_config,
        help="The application .ini config file (optional, default is '{}')".format(default_app_config)
    )
    parser.add_argument(
        "--app-name", "-n",
        default="app",
        help="The application name (optional, default is 'app')"
    )


def get_config_uri(options):
    uri = urlsplit(options.app_config)
    return urlunsplit((
        uri.scheme or 'c2cgeoportal',
        uri.netloc,
        uri.path,
        uri.query,
        options.app_name or uri.fragment
    ))


def get_appsettings(options, defaults=None):
    config_uri = get_config_uri(options)
    loader = get_config_loader(config_uri)
    loader.setup_logging()
    return loader.get_wsgi_app_settings(defaults=defaults)



# -*- coding: utf-8 -*-

# Copyright (c) 2018-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


from papyrus.xsd import XSDGenerator as PapyrusXSDGenerator
from papyrus.xsd import tag
from sqlalchemy.orm.properties import ColumnProperty
from sqlalchemy.orm.util import class_mapper
from six import BytesIO

import logging
log = logging.getLogger(__name__)


class XSDGenerator(PapyrusXSDGenerator):

    def add_class_properties_xsd(self, tb, cls):
        """ Add the XSD for the class properties to the ``TreeBuilder``. And
        call the user ``sequence_callback``. """
        mapper = class_mapper(cls)

        properties = []
        if cls.__attributes_order__:
            for attribute_name in cls.__attributes_order__:
                attr = mapper.attrs.get(attribute_name)
                if attr:
                    properties.append(attr)

            # Add other attributes
            for p in mapper.iterate_properties:
                if p not in properties:
                    properties.append(p)
        else:
            properties = mapper.iterate_properties

        for p in properties:
            if isinstance(p, ColumnProperty):
                self.add_column_property_xsd(tb, p)

        if self.sequence_callback:
            self.sequence_callback(tb, cls)

    def add_column_property_xsd(self, tb, column_property):
        column = column_property.columns[0]
        if column.foreign_keys:
            self.add_association_proxy_xsd(tb, column_property)
            return

        super().add_column_property_xsd(tb, column_property)

    # pylint: disable=no-self-use
    def add_association_proxy_xsd(self, tb, column_property):
        from c2cgeoportal_commons.models import DBSession
        column = column_property.columns[0]
        proxy = column.info['association_proxy']
        attribute = column_property.class_attribute
        cls = attribute.parent.entity
        association_proxy = getattr(cls, proxy)
        relationship_property = class_mapper(cls) \
            .get_property(association_proxy.target)
        target_cls = relationship_property.argument
        query = DBSession.query(getattr(target_cls, association_proxy.value_attr))
        attrs = {}
        if association_proxy.nullable:
            attrs["minOccurs"] = "0"
            attrs["nillable"] = "true"
        attrs["name"] = proxy
        with tag(tb, "xsd:element", attrs) as tb:
            with tag(tb, "xsd:simpleType") as tb:
                with tag(tb, "xsd:restriction",
                         {"base": "xsd:string"}) as tb:
                    for value, in query:
                        with tag(tb, "xsd:enumeration", {"value": value}):
                            pass
            self.element_callback(tb, column)

    # pylint: disable=method-hidden, no-self-use
    def element_callback(self, tb, column):
        if column.info.get('readonly'):
            with tag(tb, 'xsd:annotation'):
                with tag(tb, 'xsd:appinfo'):
                    with tag(tb, 'readonly', {'value': 'true'}):
                        pass


class XSD(object):
    def __init__(
        self,
        include_primary_keys=False,
        include_foreign_keys=False,
        sequence_callback=None,
        element_callback=None
    ):
        self.generator = XSDGenerator(
            include_primary_keys=include_primary_keys,
            include_foreign_keys=include_foreign_keys,
            sequence_callback=sequence_callback,
            element_callback=element_callback
        )

    def __call__(self, table):
        def _render(cls, system):
            request = system.get('request')
            if request is not None:
                response = request.response
                response.content_type = 'application/xml'
                io = self.generator.get_class_xsd(BytesIO(), cls)
                return io.getvalue()
        return _render



# -*- coding: utf-8 -*-

# Copyright (c) 2013-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


from c2cgeoportal_geoportal.views.proxy import Proxy
import logging
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config
import re

logger = logging.getLogger(__name__)


class Dev(Proxy):

    THEME_RE = re.compile(r'/theme/.*$')

    def __init__(self, request):
        super().__init__(request)
        self.dev_url = self.request.registry.settings['devserver_url']

    @view_config(route_name='dev')
    def dev(self):
        path = self.THEME_RE.sub('', self.request.path_info)
        if self.request.path.endswith('/dynamic.js'):
            return HTTPFound(location=self.request.route_url('dynamic', _query=self.request.params))
        else:
            return self._proxy_response('dev', "{}/{}".format(
                self.dev_url.rstrip('/'), path.lstrip('/')
            ))



# -*- coding: utf-8 -*-

# Copyright (c) 2018-2019, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

# pylint: disable=missing-docstring,attribute-defined-outside-init,protected-access


import pyramid.registry
from unittest import TestCase
from pyramid.testing import DummyRequest


def handler(request):
    return request.response


class MyRequest(DummyRequest):
    def __init__(self, path_info):
        self.path_info = path_info


class TestCacheBuster(TestCase):

    @classmethod
    def setup_class(cls):
        pass

    def test_tween(self):
        from c2cgeoportal_geoportal.lib.headers import HeadersTween

        registry = pyramid.registry.Registry()
        registry.settings = {
            'global_headers': [{
                'pattern': '^/test/.*',
                'headers': {
                    'X-Test': 'TOTO'
                }
            }]
        }
        ht = HeadersTween(handler, registry)

        request = MyRequest("/test/titi")
        response = ht(request)
        assert response.headers['X-Test'] == 'TOTO'

        request = MyRequest("/test2/titi")
        response = ht(request)
        assert 'X-Test' not in response.headers
